'use strict';

var EditBox = cc.EditBox;

if (EditBox && EditBox._EditBoxImpl) {
    var getKeyboardReturnType = function getKeyboardReturnType(type) {
        switch (type) {
            case KeyboardReturnType.DEFAULT:
            case KeyboardReturnType.DONE:
                return 'done';
            case KeyboardReturnType.SEND:
                return 'send';
            case KeyboardReturnType.SEARCH:
                return 'search';
            case KeyboardReturnType.GO:
                return 'go';
            case KeyboardReturnType.NEXT:
                return 'next';
        }
        return 'done';
    };

    var updateLabelsVisibility = function updateLabelsVisibility(editBox) {
        var displayText = editBox._impl._text;
        var textLabel = editBox._textLabel;
        var placeholderLabel = editBox._placeholderLabel;

        if (textLabel) {
            textLabel.node.active = displayText !== '';
        }
        if (placeholderLabel) {
            placeholderLabel.node.active = displayText === '';
        }
    };

    var KeyboardReturnType = EditBox.KeyboardReturnType;
    var _currentEditBoxImpl = null;

    Object.assign(EditBox.prototype, {
        editBoxEditingDidBegan: function editBoxEditingDidBegan() {
            cc.Component.EventHandler.emitEvents(this.editingDidBegan, this);
            this.node.emit('editing-did-began', this);
        },
        editBoxEditingDidEnded: function editBoxEditingDidEnded() {
            cc.Component.EventHandler.emitEvents(this.editingDidEnded, this);
            this.node.emit('editing-did-ended', this);
        },
        _updateStayOnTop: function _updateStayOnTop() {
            // baidu_game not support
        }
    });

    Object.assign(EditBox._EditBoxImpl.prototype, {
        setFocus: function setFocus() {
            this._beginEditing();
        },
        isFocused: function isFocused() {
            return this._editing;
        },
        setInputMode: function setInputMode(inputMode) {
            this._inputMode = inputMode;
        },
        _beginEditing: function _beginEditing() {
            this.createInput();
        },
        _endEditing: function _endEditing() {
            this._delegate && this._delegate.editBoxEditingDidEnded();
            this._editing = false;
        },
        createInput: function createInput() {
            // Unregister keyboard event listener in old editBoxImpl if keyboard haven't hidden.
            if (_currentEditBoxImpl !== this) {
                if (_currentEditBoxImpl) {
                    var self = this;
                    _currentEditBoxImpl._endEditing();
                    swan.offKeyboardConfirm(_currentEditBoxImpl.onKeyboardConfirmCallback);
                    swan.offKeyboardInput(_currentEditBoxImpl.onKeyboardInputCallback);
                    swan.offKeyboardComplete(_currentEditBoxImpl.onKeyboardCompleteCallback);
                    // Can't show new keyboard if the old one is shown.
                    swan.hideKeyboard({
                        success: function success(res) {
                            self.createInput(); // show new keyboard after hiding the old one.
                        },
                        fail: function fail(res) {
                            cc.warn(res.errMsg);
                        }
                    });
                    _currentEditBoxImpl = this;
                    return;
                }
                _currentEditBoxImpl = this;
            }

            var multiline = this._inputMode === EditBox.InputMode.ANY;
            var editBoxImpl = this;
            this._editing = true;

            function onKeyboardConfirmCallback(res) {
                editBoxImpl._text = res.value;
                editBoxImpl._delegate && editBoxImpl._delegate.editBoxEditingReturn && editBoxImpl._delegate.editBoxEditingReturn();
                swan.hideKeyboard({
                    success: function success(res) {},
                    fail: function fail(res) {
                        cc.warn(res.errMsg);
                    }
                });
            }

            function onKeyboardInputCallback(res) {
                if (res.value.length > editBoxImpl._maxLength) {
                    res.value = res.value.slice(0, editBoxImpl._maxLength);
                }
                if (editBoxImpl._delegate && editBoxImpl._delegate.editBoxTextChanged) {
                    if (editBoxImpl._text !== res.value) {
                        editBoxImpl._text = res.value;
                        editBoxImpl._delegate.editBoxTextChanged(editBoxImpl._text);
                        updateLabelsVisibility(editBoxImpl._delegate);
                    }
                }
            }

            function onKeyboardCompleteCallback() {
                editBoxImpl._endEditing();
                swan.offKeyboardConfirm(onKeyboardConfirmCallback);
                swan.offKeyboardInput(onKeyboardInputCallback);
                swan.offKeyboardComplete(onKeyboardCompleteCallback);
                _currentEditBoxImpl = null;
            }

            swan.showKeyboard({
                defaultValue: editBoxImpl._text,
                maxLength: editBoxImpl._maxLength,
                multiple: multiline,
                confirmHold: false, // hide keyboard mannually by swan.onKeyboardConfirm
                confirmType: getKeyboardReturnType(editBoxImpl._returnType),
                success: function success(res) {
                    editBoxImpl._delegate && editBoxImpl._delegate.editBoxEditingDidBegan && editBoxImpl._delegate.editBoxEditingDidBegan();
                    swan.onKeyboardConfirm(onKeyboardConfirmCallback);
                    swan.onKeyboardInput(onKeyboardInputCallback);
                    swan.onKeyboardComplete(onKeyboardCompleteCallback);
                },
                fail: function fail(res) {
                    cc.warn(res.errMsg);
                    editBoxImpl._endEditing();
                }
            });
        }
    });
}