'use strict';

var inputManager = _cc.inputManager;
var renderer = cc.renderer;
var game = cc.game;
var _frameRate = 60;

Object.assign(game, {
    setFrameRate: function setFrameRate(frameRate) {
        _frameRate = frameRate;
        swan.setPreferredFramesPerSecond(frameRate);
    },
    getFrameRate: function getFrameRate() {
        return _frameRate;
    },
    _runMainLoop: function _runMainLoop() {
        var self = this,
            _callback,
            config = self.config,
            director = cc.director,
            skip = true,
            frameRate = config.frameRate;

        cc.debug.setDisplayStats(config.showFPS);

        _callback = function callback() {
            if (!self._paused) {
                self._intervalId = window.requestAnimFrame(_callback);
                director.mainLoop();
            }
        };

        self._intervalId = window.requestAnimFrame(_callback);
        self._paused = false;
    },
    _initRenderer: function _initRenderer() {
        // Avoid setup to be called twice.
        if (this._rendererInitialized) return;

        var el = this.config.id,
            width = void 0,
            height = void 0,
            localCanvas = void 0,
            localContainer = void 0;

        this.container = localContainer = document.createElement("DIV");
        this.frame = localContainer.parentNode === document.body ? document.documentElement : localContainer.parentNode;

        if (cc.sys.browserType === cc.sys.BROWSER_TYPE_BAIDU_GAME_SUB) {
            localCanvas = window.sharedCanvas || swan.getSharedCanvas();
        } else {
            localCanvas = canvas;
        }
        this.canvas = localCanvas;

        this._determineRenderType();
        // WebGL context created successfully
        if (this.renderType === this.RENDER_TYPE_WEBGL) {
            var opts = {
                'stencil': true,
                // MSAA is causing serious performance dropdown on some browsers.
                'antialias': cc.macro.ENABLE_WEBGL_ANTIALIAS,
                'alpha': cc.macro.ENABLE_TRANSPARENT_CANVAS
            };

            opts['preserveDrawingBuffer'] = true;

            renderer.initWebGL(localCanvas, opts);
            this._renderContext = renderer.device._gl;

            // Enable dynamic atlas manager by default
            if (!cc.macro.CLEANUP_IMAGE_CACHE && dynamicAtlasManager) {
                dynamicAtlasManager.enabled = true;
            }
        }
        if (!this._renderContext) {
            this.renderType = this.RENDER_TYPE_CANVAS;
            // Could be ignored by module settings
            renderer.initCanvas(localCanvas);
            this._renderContext = renderer.device._ctx;
        }

        this.canvas.oncontextmenu = function () {
            if (!cc._isContextMenuEnable) return false;
        };

        this._rendererInitialized = true;
    },
    _initEvents: function _initEvents() {
        var win = window,
            hiddenPropName;

        // register system events
        if (this.config.registerSystemEvent) inputManager.registerSystemEvent(this.canvas);

        if (typeof document.hidden !== 'undefined') {
            hiddenPropName = "hidden";
        } else if (typeof document.mozHidden !== 'undefined') {
            hiddenPropName = "mozHidden";
        } else if (typeof document.msHidden !== 'undefined') {
            hiddenPropName = "msHidden";
        } else if (typeof document.webkitHidden !== 'undefined') {
            hiddenPropName = "webkitHidden";
        }

        var hidden = false;

        function onHidden() {
            if (!hidden) {
                hidden = true;
                game.emit(game.EVENT_HIDE);
            }
        }
        function onShown() {
            if (hidden) {
                hidden = false;
                game.emit(game.EVENT_SHOW);
            }
        }

        if (hiddenPropName) {
            var changeList = ["visibilitychange", "mozvisibilitychange", "msvisibilitychange", "webkitvisibilitychange", "qbrowserVisibilityChange"];
            for (var i = 0; i < changeList.length; i++) {
                document.addEventListener(changeList[i], function (event) {
                    var visible = document[hiddenPropName];
                    // QQ App
                    visible = visible || event["hidden"];
                    if (visible) onHidden();else onShown();
                });
            }
        } else {
            win.addEventListener("blur", onHidden);
            win.addEventListener("focus", onShown);
        }

        if (navigator.userAgent.indexOf("MicroMessenger") > -1) {
            win.onfocus = onShown;
        }

        if (cc.sys.browserType !== cc.sys.BROWSER_TYPE_BAIDU_GAME_SUB) {
            swan.onShow && swan.onShow(onShown);
            swan.onHide && swan.onHide(onHidden);
        }

        if ("onpageshow" in window && "onpagehide" in window) {
            win.addEventListener("pagehide", onHidden);
            win.addEventListener("pageshow", onShown);
            // Taobao UIWebKit
            document.addEventListener("pagehide", onHidden);
            document.addEventListener("pageshow", onShown);
        }

        this.on(game.EVENT_HIDE, function () {
            game.pause();
        });
        this.on(game.EVENT_SHOW, function () {
            game.resume();
        });
    }
});